/*
** Copyright (c) 2000 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*************************************************************************
** This file implements the main routine for TWS the Tcl Web Server.
**
** @(#) $Id: main.c,v 1.5 2000/12/29 13:30:44 drh Exp $
*/
static char const rcsid[] = "@(#) $Id: main.c,v 1.5 2000/12/29 13:30:44 drh Exp $";
#include <tk.h>

/*
** We will be linking against all of these extensions.
*/
extern int Sqlite_Init(Tcl_Interp*);
extern int Zvfs_Init(Tcl_Interp*);
extern int Zvfs_Mount(Tcl_Interp*, const char*, const char *);
extern int Tws_Init(Tcl_Interp*);

/*
** This is the event loop
*/
static char zWaitForever[] = 
  "while 1 {vwait forever}"
;

/*
** The name of the fake directory where we mount all the files that
** are contained in the ZIP archive that is appended to the executable.
*/
#define ZVFS "/-builtin-"

/*
** This routine runs first.  
*/
int main(int argc, char **argv){
  Tcl_Interp *interp;
  char *args;
  char buf[100];

  /* Create a Tcl interpreter
  */
  Tcl_FindExecutable(argv[0]);
  interp = Tcl_CreateInterp();
  if( Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1)==0 ){
    return 1;
  }
  args = Tcl_Merge(argc-1, argv+1);
  Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
  ckfree(args);
  sprintf(buf, "%d", argc-1);
  Tcl_SetVar(interp, "zvfs_root", ZVFS, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "argv0", argv[0], TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);

  /* We have to initialize the virtual filesystem before calling
  ** Tcl_Init().  Otherwise, Tcl_Init() will not be able to find
  ** its startup script files.
  */
  Zvfs_Init(interp);
  Zvfs_Mount(interp, Tcl_GetNameOfExecutable(), ZVFS);
  Tcl_SetVar2(interp, "env", "TCL_LIBRARY", ZVFS "/tcl", TCL_GLOBAL_ONLY);

  /* Initialize Tcl
  */
  if( Tcl_Init(interp) ) goto error_out;

  /* Start up all extensions.
  */
  if( Tws_Init(interp) ) goto error_out;
  Tcl_StaticPackage(interp, "Tws", Tws_Init, 0);
  if( Sqlite_Init(interp) ) goto error_out;
  Tcl_StaticPackage(interp, "Sqlite", Sqlite_Init, 0);

  /* After all extensions are registered, start up the
  ** program by running /-builtin-/main.tcl.
  */
  if( Tcl_Eval(interp, "source " ZVFS "/main.tcl") ) goto error_out;
  Tcl_Eval(interp, zWaitForever);
  exit(0);

error_out:
  fprintf(stderr, "%s\n", Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY));
  exit(1);
}
