# @(#) $Id: main.tcl,v 1.9 2001/02/17 18:56:27 drh Exp $
#
# This script runs first.  To build a standalone application out of
# TWS, modify this script to initialize the application.
#

# Select default execution parameters.
#
# By default, TWS will chroot to the directory that contains the
# executable.  To disable this feature use the "-chroot /" command-line
# option.
#
# The default user will be the owner of the tws.conf file in the same
# directory as the executable.  If there is no tws.conf file in that
# directory, then the default user is selected from a set of common
# unprivileged user names: nobody guest httpd http www.  If none of these
# users exist, then a user-id of 1000 is used.
#
set ::tws::httpd_port 80
set ::tws::chrootjail [file dir [info nameofexe]]
if {[catch {file attr $::tws::chrootjail/tws.conf -owner} _owner]==0} {
  set ::tws::username $_owner
  unset _owner
} else {
  set ::tws::username nobody
}


# Parse the command-line arguments
#
for {set i 0} {$i<[llength $argv]} {incr i} {
  switch -glob -- [lindex $argv $i] {
    -port {
      incr i
      set ::tws::httpd_port [lindex $argv $i]
    }
    -chroot {
      incr i
      set ::tws::chrootjail [lindex $argv $i]
    }
    -user {
      incr i
      set ::tws::username [lindex $argv $i]
    }
    default {
      puts stderr "unknown option: [lindex $argv $i]"
      puts ""
      puts "Options:"
      puts "   -chroot PATH  Put the server in a chroot jail."
      puts "   -port N       Listen on TCP port N.  Default=80."
      puts "   -user NAME    Change user ID to NAME after opening TCP port"
      exit 1
    }
  }
}

# Get the numeric user and group IDs that the TWS server will use
# while it is running.  We have to do this first, before doing
# the chroot because after the chroot we will no longer have access
# to /etc/passwd.
#
# We will attempt to change to an unprivileged user even if no 
# alternative user is specified on the command line.  (This is
# an effort to implement security by default.)  If no appropriate
# user is found, then guess.
#
if {$::tws::username!=""} {
  set ::tws::userid [::tws::getuserid $::tws::username]
} else {
  set ok 0
  foreach uname {nobody guest httpd http www} {
    if {[catch {::tws::getuserid $uname} ::tws::userid]==0} {
      set ::tws::username $uname
      set ok 1
      break
    }
  }
  if {!$ok} {
    set ::tws::userid {1000 1000}
    set ::tws::username user-1000
  }
  unset ok
  unset uname
}

# Move into our chroot jail before doing anything else.  Note that we
# have to remount the ZIP filesystem after changing the root directory.
# Note also that we will be unable to remount the ZIP filesystem if
# the TWS executable is outside of the root jail.
#
if {$::tws::chrootjail=="/"} {
  set ::tws::chrootjail {}
}
if {$::tws::chrootjail!=""} {
  if {![::tws::chroot $::tws::chrootjail]} {
    set ::tws::chrootjail {}
  } else {
    set name_of_executable [info nameofexe]
    zvfs::unmount $name_of_executable
    regsub -all {[][()*+.$\\]} $::tws::chrootjail {\\&} _reg
    regsub ^$_reg $name_of_executable {} name_of_executable
    unset _reg
    zvfs::mount $name_of_executable $zvfs_root
  }
}

# Load the TCL code that implements the TWS server
#
source $zvfs_root/tws.tcl
set D(namespace) {}
::tws::require $zvfs_root/config/common.tcl

# Start the server and become an unprivileged user.  We have to wait
# to the change users until after the server has started because we
# might need to be the superuser in order to open the server TCP port.
#
::tws::start $::tws::httpd_port
catch {eval ::tws::setuser $::tws::userid}
