# Copyright (c) 2000 D. Richard Hipp
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   drh@hwaci.com
#   http://www.hwaci.com/drh/
#
###############################################################################
#
# Handlers for configuring the TWS web server using a web browser.
#
# @(#) $Id: common.tcl,v 1.4 2001/02/17 18:56:27 drh Exp $
#

# All of this configuration code runs in its own namespace.
#
namespace eval ::twscfg {

# Import everything from the ::tws:: namespace.  There are a lot
# of convenient tools there.
#
namespace import -force ::tws::*

# The names and properties of all documents
#
variable page_list {
  main       1 Intro        {Introduction}             index
  dispatcher 1 Dispatch     {Dispatchers}              pw/dispatcher
  dedit      0 {}           {Edit Dispatcher}          pw/dedit
  mime       1 MIME         {MIME Types}               pw/mime
  medit      0 {}           {Edit MIME Type}           pw/medit
  auth       1 Access       {Access Control}           pw/auth
  aedit      0 {}           {Edit Access Control}      pw/aedit
  user       1 Users        {Users}                    pw/user
  uedit      0 {}           {Edit User}                pw/uedit
  shutdown   1 Shutdown     {Server Shutdown}          pw/shutdown
}

# Generate a document header.
#
proc header {id} {
  foreach {tid menu_flag abbrev title url} ${::twscfg::page_list} {
    if {$tid==$id} break
  }
  upvar 1 S S D D
  set S(id) $id
  set S(title) $title
  set S(url) $D(prefix)/$url

  # Color 1 is the main background.  Color 2 is the interior of the
  # notebook pages.
  set S(bg1) \"#5b597b\"
  set S(fg1) \"white\"
  set S(bg2) \"#b9cfff\"
  set S(fg2) \"black\"

  # IE does not honor a <font color=> inside an <a href=>..</a>.  So
  # for IE we have to attach a style attribute to the <a> to change
  # its color.  The following is that attribute.
  set fg1style " style=\"color: [string trim $S(fg1) \"]\""

  return [hsubst {<html>
<head>
  <title><$S(title)></title>
</head>
<body bgcolor=<$S(bg1)> text=<$S(fg2)>>
<font color=<$S(fg1)>>
<table width="100%">
<tr>
<td align="left">
<font size="6" color=<$S(fg1)>><b>TWS Configuration</b></font>
</td>
<td align="right">
<font size="5" color=<$S(fg1)>><$title></font>
</td>
</tr>
</table>
<table width="100%" border="0" cellpadding="0" cellspacing="0">
<tr>
<td valign="top">
  <table hspace="0" cellpadding="5" cellspacing="0" border="0">
  <tr><td></td></tr>
<%
  foreach {tid mflag abbrev mtitle url} ${::twscfg::page_list} {
    if {!$mflag} continue
    if {$tid==$id} {
      render "  <tr bgcolor=$S(bg2)><td align=\"right\">"
      render "<font color=$S(fg2)><b>$abbrev</b></font>"
    } else {
      render "  <tr><td align=\"right\"><a href=\"$D(prefix)/$url\"$fg1style>"
      render "<font color=$S(fg1)><b>$abbrev</b></font></a>"
    }
    render "&nbsp;&nbsp;</td></tr>\n"
  }
%>  
  <tr><td height="300"></td></tr>
  </table>
</td>
<td bgcolor=<$S(bg2)>>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td bgcolor=<$S(bg2)> valign="top" width="100%">&nbsp;<br/>
<font color=<$S(fg2)>>
}]
}

# Generate a document footer
#
proc footer {args} {
  upvar 1 S S
  return [hsubst {<br clear="both"></font>
</td>
<td bgcolor=<$S(bg2)>>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
</tr>
</table>
</body>
</html>}]
}

# Generate a link to the given document page
#
proc link {title id {query {}}} {
  foreach {tid menu_flag abbrev ttitle url} ${::twscfg::page_list} {
    if {$tid==$id} break
  }
  upvar 1 D D
  if {$query!=""} {set query "?$query"}
  return "<a href=\"$D(prefix)/$url$query\">$title</a>"
}

# Compute the name of the configuration file.
#
# We first look for a file named "tws.conf" in the same directory
# as the executable.  But if the executable is not inside the
# chroot jail (meaning that the tws.conf file would be inaccessible)
# then choose a file named "tws.conf" at the root of the
# chroot jail instead.
#
proc conffile {} {
  if {[info exists ::twscfg::config_filename]} {
    return $::twscfg::config_filename
  }
  set name [file dir [info nameofexe]]/tws.conf
  if {[info exists ::tws::chrootjail] && $::tws::chrootjail!=""} {
    regsub -all {[][()*+.$\\]} $::tws::chrootjail {\\&} reg
    if {[regexp ^$reg $name]==0} {
      set name $::tws::chrootjail/tws.conf
    }
  }
  set ::twscfg::config_filename $name
  return $name
}

# Load the configuration file.  The name of the file is "tws.conf"
# and it should be located in the same directory as the executable.
# If tws.conf cannot be found, do a default configuration.
#
proc load_config {} {
  set conffile [conffile]
  ::tws::chroot_fixup conffile
  if {![file exists $conffile]} {
    default_config
    return
  }
  catch {file attributes $conffile -permissions 0400}
  if {[catch {source $conffile}]} {
    default_config
    return
  }
  catch {file attributes $conffile -permissions 0000}
  catch {file attributes $conffile -owner [lindex $::tws::userid 0]}
}

# Set up a default configuration.  This happens when the
# configuration file does not exist or cannot be read.
#
proc default_config {} {
  ::tws::dispatcher localhost /config [list ${::zvfs_root}/config {} ::twscfg]
  set ::tws::extension(tws)   {dynamic text/html}
  set ::tws::extension(html)  {static text/html}
  set ::tws::extension(htm)   {static text/html}
  set ::tws::extension(gif)   {static image/gif}
  set ::tws::extension(jpeg)  {static image/jpeg}
  set ::tws::extension(jpg)   {static image/jpeg}
  set ::tws::extension(png)   {static image/png}
  set ::tws::extension(txt)   {static text/plain}
  set ::tws::extension(gz)    {static application/x-unknown}
  set ::tws::extension(zip)   {static application/x-unknown}
  set ::tws::extension(bz2)   {static application/x-unknown}
  set ::tws::extension(exe)   {static application/x-unknown}
  set ::tws::realm(admin) {{} 0 {TWS Administration}}
}

# Make arrangements to write the configuration file the next
# time we are idle.
#
variable write_config_pending 0
proc write_config {} {
  if {$::twscfg::write_config_pending} return
  set ::twscfg::write_config_pending 1
  after idle ::twscfg::write_config_callback
}

# This routine does the actual work of writing the configuration file.
#
proc write_config_callback {} {
  set ::twscfg::write_config_pending 0
  set cf [conffile]
  ::tws::chroot_fixup cf
  catch {file attributes $cf -permissions 0600}
  catch {file attributes $cf -hidden 0}
  if {[catch {open $cf w} fd]} return
  foreach dispatch [::tws::dispatcher_list] {
    puts $fd "::tws::dispatcher $dispatch"
  }
  foreach v {extension realm user userpw} {
    puts $fd "catch {unset ::tws::$v}"
    foreach a [array names ::tws::$v] {
      puts $fd "set [list ::tws::${v}($a)] [list [set ::tws::${v}($a)]]"
    }
  }
  close $fd
  catch {file attributes $cf -permissions 0000}
  catch {file attributes $cf -hidden 1}
}

# This is the end of the "namespace eval twscfg"
}

# Register the configuration pages with TWS.
#
::twscfg::load_config
::twscfg::write_config
