#!/usr/bin/tclsh
#
# Run this script over the TWS source files and generate
# a document that describes the various TCL commands that
# are implemented there.
#
# Documentation on TCL commands is contained in comments
# that begin with "tclcmd:"
#
puts {
  set rcsid {$Id: make_tcl_doc.tcl,v 1.5 2001/02/18 01:06:11 drh Exp $}
}

set cnt 1
proc ProcessFile {filename} {
  set fd [open $filename r]
  set text [read $fd [file size $filename]]
  close $fd
  global Msg Title Prototype Cnt cnt Label
  set op {}
  set comment .
  foreach lx [split $text \n] {
    set line [string trim $lx]
    if {[regexp {^ *[/*#]+ tclcmd: } $line]} {
      set comment [string index $line 0]
      set op [lindex $line 2]
      set Prototype($op) [lrange $line 2 end]
      set Cnt($op) $cnt
      regsub -all {[^a-zA-Z0-9]} $op _ op2
      set Label($op) $op2
      incr cnt
      continue
    }
    if {$op==""} continue
    if {[string index $line 0]!=$comment} {
      set op {}
      continue
    }
    if {[regexp {^ *[/*#]+ title: } $line]} {
      set Title($op) [lrange $line 2 end]
      continue
    }
    regsub {^ *[/*#]+} $lx {} line
    if {[string trim $line]==""} {
      set line <p>
    }
    append Msg($op) $line\n
  }
}

set FileList {
  tws.c
  tws.tcl
}

set path [lindex $argv 0]

foreach f $FileList {
  ProcessFile $path/$f
}

puts {
source common.tcl
header "New TCL Commands"
}

puts "puts \173"
puts {
<p>
TWS implements several new TCL command that are useful for generating
dynamic content.  This document describes the new commands.</p>

<p>All new TCL commands are in the <b>::tws::</b> namespace.  
This is the default namespace for executing dynamic content in TWS
and so the usual case is that all the new commands can invoked
exactly as shown below.
But if you specify an alternative namespace in the dispatcher, then
you must either import the new commands into the alternative
namespace or specify the full namespace path each time you invoke the
command.</p>

<p>The following list summarizes the available commands.</p>

<blockquote>
<table>}

foreach op [lsort [array names Msg]] {
  puts "<tr><td><a href=\"#idx_$Label($op)\">$op</td>"
  puts "    <td>&nbsp;&nbsp;&nbsp;</td>"
  puts "    <td>$Title($op)</td></tr>"
}
puts {</table>
</blockquote>

<p>
A more detailed description of each of the new TCL commands is 
given below.
All of these descriptions are extracted from comments in the source code.
See the actual source code for additional information.
</p>
}

foreach op [lsort [array names Msg]] {
  puts "<a name=\"idx_$Label($op)\">"
  puts "<h2>$op - $Title($op)</h2>\n"
  # puts "<table cellspacing=\"20\">"
  # puts "<tr><td align=\"right\" valign=\"top\">Synopsis:</td>"
  # puts "<td align=\"left\" valign=\"top\">$Title($op)</td></tr>"
  # puts "<tr><td align=\"right\" valign=\"top\">Syntax:</td>\n"
  # puts "<td align=\"left\" valign=\"top\">::tws::$Prototype($op)</td></tr>"
  # puts "<tr><td align=\"right\" valign=\"top\">Description:</td>"
  # puts "<td colspan=\"2\">$Msg($op)</td></tr>"
  # puts "</table>"
  puts "<p>Syntax: <b>$Prototype($op)</b></p>"
  puts "<p>$Msg($op)</p>"
}

puts "\175\nfooter"
